package com.sun.org.apache.xml.internal.security.utils;

import java.util.Collections;
import java.util.WeakHashMap;
import java.util.concurrent.LinkedBlockingDeque;
import java.util.Map;
import java.lang.ref.WeakReference;
import java.util.concurrent.BlockingQueue;

public abstract class WeakObjectPool<T, E extends Throwable>
{
    private static final Integer MARKER_VALUE;
    private final BlockingQueue<WeakReference<T>> available;
    private final Map<T, Integer> onLoan;
    
    protected WeakObjectPool() {
        this.available = new LinkedBlockingDeque();
        this.onLoan = Collections.synchronizedMap(new WeakHashMap());
    }
    
    protected abstract T createObject() throws E, Throwable;
    
    public T getObject() throws E, Throwable {
        Object o = null;
        WeakReference<Object> weakReference;
        do {
            weakReference = (WeakReference)this.available.poll();
        } while (weakReference != null && (o = weakReference.get()) == null);
        if (o == null) {
            o = this.createObject();
        }
        this.onLoan.put(o, WeakObjectPool.MARKER_VALUE);
        return o;
    }
    
    public boolean repool(T referent) {
        if (referent != null && this.onLoan.containsKey(referent)) {
            synchronized (referent) {
                if (this.onLoan.remove(referent) != null) {
                    return this.available.offer(new WeakReference(referent));
                }
            }
        }
        return false;
    }
    
    static {
        MARKER_VALUE = Integer.valueOf(Integer.MAX_VALUE);
    }
}
